// ignore_for_file: cancel_subscriptions, public_member_api_docs
// ignore_for_file: non_constant_identifier_names, empty_catches
// ignore_for_file: avoid_dynamic_calls

import 'dart:async';

import 'package:flutter_blue_plus/flutter_blue_plus.dart';
import 'package:healthrian_ble_support_for_cardioclick/'
    'healthrian_ble_support_for_cardioclick.dart';
import 'package:healthrian_common_support/'
    'healthrian_common_support.dart';
import 'package:healthrian_ble_support_for_wearecg12/'
    'healthrian_ble_support_for_wearecg12.dart' as wearecg12;

class BLEManager {
  factory BLEManager() {
    return _instance;
  }

  BLEManager._internal() {}

  static final BLEManager _instance = BLEManager._internal();

  int get samplingRate {
    switch (version) {
      case Versions.v1:
        return _v1.samplingRate;
      case Versions.v2:
        return _v2.samplingRate;
    }
  }

  BluetoothDevice? get device {
    switch (version) {
      case Versions.v1:
        return _v1.device;
      case Versions.v2:
        return _v2.device;
    }
  }

  set device(BluetoothDevice? value) {
    switch (version) {
      case Versions.v1:
        _v1.device = value;
        break;
      case Versions.v2:
        _v2.device = value;
        break;
    }
  }

  Versions version = Versions.v1;
  late final wearecg12.BLEManager _v1 = wearecg12.BLEManager.CardioClick()
    ..scanFilter = (device) => device.name.isCardioClick;
  final BLEManagerV2 _v2 = BLEManagerV2();

  get streams {
    switch (version) {
      case Versions.v1:
        return _v1.streams;
      case Versions.v2:
        return _v2.streams;
    }
  }

  bool Function(BluetoothDevice device) get scanFilter {
    switch (version) {
      case Versions.v1:
        return _v1.scanFilter;
      case Versions.v2:
        return _v2.scanFilter;
    }
  }

  set scanFilter(bool Function(BluetoothDevice device) value) {
    switch (version) {
      case Versions.v1:
        _v1.scanFilter = value;
        break;
      case Versions.v2:
        _v2.scanFilter = value;
        break;
    }
  }

  Future<bool> connectDevice({String preferredDeviceId = ''}) async {
    switch (version) {
      case Versions.v1:
        return await _v1.connectDevice(preferredDeviceId: preferredDeviceId);
      case Versions.v2:
        return await _v2.connectDevice(preferredDeviceId: preferredDeviceId);
    }
  }

  /// [disconnectDevice] disconnects all devices
  /// whose name start with `C-Click`
  Future<void> disconnectDevice() async {
    switch (version) {
      case Versions.v1:
        await _v1.disconnectDevice();
        break;
      case Versions.v2:
        await _v2.disconnectDevice();
        break;
    }
  }

  /// [registerCharacteristics] registers all characteristics,
  /// defined in [Characteristics].
  ///
  /// The result is registered in a map structure [points].
  Future<void> registerCharacteristics() async {
    switch (version) {
      case Versions.v1:
        await _v1.registerCharacteristics();
        break;
      case Versions.v2:
        await _v1.registerCharacteristics();
        break;
    }
  }

  /// [initializeStreams] is used to reset measurement.
  ///
  /// Once calling it,
  /// all measurement streams become unobservable.
  Future<void> initializeStreams() async {
    switch (version) {
      case Versions.v1:
        await _v1.initializeStreams();
        break;
      case Versions.v2:
        await _v2.initializeStreams();
        break;
    }
  }

  Future<void> startMeasurementEcg() async {
    switch (version) {
      case Versions.v1:
        await _v1.startMeasurementEcg();
        break;
      case Versions.v2:
        await _v2.startMeasurementEcg();
        break;
    }
  }

  /// [pauseMeasurementEcg] pause measurements for Ecg.
  Future<void> pauseMeasurementEcg() async {
    switch (version) {
      case Versions.v1:
        await _v1.pauseMeasurementEcg();
        break;
      case Versions.v2:
        await _v2.pauseMeasurementEcg();
        break;
    }
  }

  /// [stopMeasurementEcg] sends stop_ecg_packet to device.
  Future<void> stopMeasurementEcg() async {
    switch (version) {
      case Versions.v1:
        await _v1.stopMeasurementEcg();
        break;
      case Versions.v2:
        await _v2.stopMeasurementEcg();
        break;
    }
  }

  ///////////////////////////////////////////////////////////////
  int get maxBufferStreamEcg {
    switch (version) {
      case Versions.v1:
        return _v1.maxBufferStreamEcg;
      case Versions.v2:
        return _v2.maxBufferStreamEcg;
    }
  }

  set maxBufferStreamEcg(int value) {
    switch (version) {
      case Versions.v1:
        _v1.maxBufferStreamEcg = value;
        break;
      case Versions.v2:
        _v2.maxBufferStreamEcg = value;
        break;
    }
  }

  int get lengthBufferStreamEcg {
    switch (version) {
      case Versions.v1:
        return _v1.lengthBufferStreamEcg;
      case Versions.v2:
        return _v2.lengthBufferStreamEcg;
    }
  }

  set lengthBufferStreamEcg(int value) {
    switch (version) {
      case Versions.v1:
        _v1.lengthBufferStreamEcg = value;
        break;
      case Versions.v2:
        _v2.lengthBufferStreamEcg = value;
        break;
    }
  }

  int get screenBufferStreamingEcg {
    switch (version) {
      case Versions.v1:
        return _v1.screenBufferStreamingEcg;
      case Versions.v2:
        return _v2.screenBufferStreamingEcg;
    }
  }

  set screenBufferStreamingEcg(int value) {
    switch (version) {
      case Versions.v1:
        _v1.screenBufferStreamingEcg = value;
        break;
      case Versions.v2:
        _v2.screenBufferStreamingEcg = value;
        break;
    }
  }

  int get refreshBufferEcg {
    switch (version) {
      case Versions.v1:
        return _v1.refreshBufferEcg;
      case Versions.v2:
        return _v2.refreshBufferEcg;
    }
  }

  set refreshBufferEcg(int value) {
    switch (version) {
      case Versions.v1:
        _v1.refreshBufferEcg = value;
        break;
      case Versions.v2:
        _v2.refreshBufferEcg = value;
        break;
    }
  }

  bool get isFilteredEcg {
    switch (version) {
      case Versions.v1:
        return _v1.isFilteredEcg;
      case Versions.v2:
        return _v2.isFilteredEcg;
    }
  }

  set isFilteredEcg(bool value) {
    switch (version) {
      case Versions.v1:
        _v1.isFilteredEcg = value;
        break;
      case Versions.v2:
        _v2.isFilteredEcg = value;
        break;
    }
  }

  bool get isCompressedEcg {
    switch (version) {
      case Versions.v1:
        return _v1.isCompressedEcg;
      case Versions.v2:
        return _v2.isCompressedEcg;
    }
  }

  set isCompressedEcg(bool value) {
    switch (version) {
      case Versions.v1:
        _v1.isCompressedEcg = value;
        break;
      case Versions.v2:
        _v2.isCompressedEcg = value;
        break;
    }
  }

  int get skipSample {
    switch (version) {
      case Versions.v1:
        return _v1.skipSample;
      case Versions.v2:
        return _v2.skipSample;
    }
  }

  set skipSample(int value) {
    switch (version) {
      case Versions.v1:
        _v1.skipSample = value;
        break;
      case Versions.v2:
        _v2.skipSample = value;
        break;
    }
  }

  int get skipPacket {
    switch (version) {
      case Versions.v1:
        return _v1.skipPacket;
      case Versions.v2:
        return _v2.skipPacket;
    }
  }

  set skipPacket(int value) {
    switch (version) {
      case Versions.v1:
        _v1.skipPacket = value;
        break;
      case Versions.v2:
        _v2.skipPacket = value;
        break;
    }
  }

  void Function(Channels, DataFilter)? get callback {
    switch (version) {
      case Versions.v1:
        return _v1.callback;
      case Versions.v2:
        return _v2.callback;
    }
  }

  set callback(void Function(Channels, DataFilter)? value) {
    switch (version) {
      case Versions.v1:
        _v1.callback = value;
        break;
      case Versions.v2:
        _v2.callback = value;
        break;
    }
  }

  int get periodCallback {
    switch (version) {
      case Versions.v1:
        return _v1.periodCallback;
      case Versions.v2:
        return _v2.periodCallback;
    }
  }

  set periodCallback(int value) {
    switch (version) {
      case Versions.v1:
        _v1.periodCallback = value;
        break;
      case Versions.v2:
        _v2.periodCallback = value;
        break;
    }
  }

  void Function(Channels, DataFilter)? get callbackSecond {
    switch (version) {
      case Versions.v1:
        return _v1.callbackSecond;
      case Versions.v2:
        return _v2.callbackSecond;
    }
  }

  set callbackSecond(void Function(Channels, DataFilter)? value) {
    switch (version) {
      case Versions.v1:
        _v1.callbackSecond = value;
        break;
      case Versions.v2:
        _v2.callbackSecond = value;
        break;
    }
  }

  int get periodCallbackSecond {
    switch (version) {
      case Versions.v1:
        return _v1.periodCallbackSecond;
      case Versions.v2:
        return _v2.periodCallbackSecond;
    }
  }

  set periodCallbackSecond(int value) {
    switch (version) {
      case Versions.v1:
        _v1.periodCallbackSecond = value;
        break;
      case Versions.v2:
        _v2.periodCallbackSecond = value;
        break;
    }
  }

  DataFilter get dataFilter {
    switch (version) {
      case Versions.v1:
        return _v1.dataFilter;
      case Versions.v2:
        return _v2.dataFilter;
    }
  }

  Channels? get lastChannel {
    switch (version) {
      case Versions.v1:
        return _v1.lastChannel;
      case Versions.v2:
        return _v2.lastChannel;
    }
  }

  set lastChannel(Channels? value) {
    switch (version) {
      case Versions.v1:
        _v1.lastChannel = value;
        break;
      case Versions.v2:
        _v2.lastChannel = value;
        break;
    }
  }
}
